#region File Header
// <copyright company="Bosch Security Systems GmbH">
//
// Authors:
//    Hubert Streidl<Hubert.Streidl@de.bosch.com>
//    Andreas Wetterich
//
// $Revision: 1.5 $
//
// </copyright>
#endregion

using System;
using System.Diagnostics;

using FSM5000FSI;
using FSM5000FSIAPI.Version3;

namespace FSISample1
{
   /// <summary>
   /// The class demonstrates the 
   ///  - instantiation,
   ///  - configuration,
   ///  - tracing,
   ///  - configuration receiving and
   ///  - event receiving
   /// of the FSI SDK.
   /// 
   /// To use the sample you have to copy the DLLs delived as FSM5000FSI 
   /// to a subfolder with the name sdk. Otherwise you have errors about 
   /// missing references.
   /// 
   /// Configuration:
   /// Adapt the default config values in Main() according to your own  
   /// network configuration.
   /// (The default values in this sample application correspond to the 
   /// example values in the FPA-5000 Networking Guide document.)
   /// </summary>
   class FSISample1
   {    
      /// <summary>
      /// Receive the traces from the FSI and output the traces to console
      /// In real application it is very useful to write the traces to files
      /// for suppport issues
      /// </summary>
      class TraceConsoleWritter : TraceListener
      {
         public override void Write(string message)
         {
            Console.Out.Write(message);
         }

         public override void WriteLine(string message)
         {
            Console.Out.WriteLine(message);
         }
      }

      /// <summary>
      /// The Config Listener receives all information about the configuration 
      /// received from the panel network. The information must be stored in real application
      /// to know the devices from the panel network and the possible commands.
      /// </summary>
      class ConfigListener : IPanelConfigListener
      {
         public void SetItemType(ItemType devItemType)
         {
            Console.Out.WriteLine("Item type " + devItemType.FunctionalType + " (" + devItemType.Description + ")");
         }

         public void SetItem(Item devItem)
         {
            Console.Out.WriteLine("Item  (" + devItem.SIID + ", " + devItem.FunctionalType + " (" + devItem.Label + ")");
         }

         public void RemoveItem(SIID address)
         {
            Console.Out.WriteLine("SI " + address + " removed.");
         }
      }

      /// <summary>
      /// The State Listener receive all states for devices send by the panel network.
      /// 
      /// </summary>
      class StateListener : IStateListener
      {
         public void SetItemState(ItemState devItemState)
         {
            Console.Out.WriteLine("State " + devItemState.SIID + " = " + devItemState.LogicalState);          
         }

         public void SetMPNetTime(DateTime mpNetTime)
         {
            Console.Out.WriteLine("Time is " + mpNetTime + ".");
         }
      }

      /// <summary>
      /// The main function to start the program
      /// </summary>
      /// <param name="args"></param>
      static void Main(string[] args)
      {
         //Create configuration for FSI 
         FSIConfig cfgFSI = new FSIConfig();

         // ! --- start here to replace values with the values from your own configuration ! ---
         cfgFSI.MPNetGroup             = 2;
         cfgFSI.MPNetNode              = 1;
         cfgFSI.PNA                    = 33;
         cfgFSI.LocalIPAddress         = new byte[] { 192, 168, 1, 33 };
         cfgFSI.RepositoryLocation     = ".\\repository";

         //  AdvancedFSIConfig is necessary for IP-Multicast settings
         AdvancedFSIConfig advancedFSIConfig = new AdvancedFSIConfig();
         advancedFSIConfig.IsIPMulticast = true;
         advancedFSIConfig.MulticastAddress = new byte[] { 239, 192, 0, 1 };
         advancedFSIConfig.PortNumber = 25001;
         advancedFSIConfig.NetstackTraceLevel = SourceLevels.Error;
         cfgFSI.AdvancedFSIConfig = advancedFSIConfig;
         // ! --- finish here to replace values with the values from your own configuration ! ---

         //Create the instance of the FSI
         FSIController fsi = FSIController.GetInstance();

         //Register the listeners 
         fsi.AddListener(new ConfigListener());
         fsi.AddListener(new StateListener());

         //Activate tracing
         TraceConsoleWritter outputter = new TraceConsoleWritter();
         fsi.SetTraceLevel(SourceLevels.Warning);  //for support issues please set the level to SourceLevels.Verbose

         //Startup with configuration
         Result result = fsi.Startup(cfgFSI);

         Console.Out.WriteLine("Startup=" + result);                
         
         //user interaction to finish the application
         Console.Out.WriteLine("======================================================================");
         Console.Out.WriteLine("Press e to end the sample application");
         Console.Out.WriteLine("======================================================================");

         ConsoleKeyInfo input;
         do
         {
            input = Console.ReadKey();
         }
         while (input.Key != ConsoleKey.E);
        
         
         //shutdown the sample application
         fsi.Shutdown();           
        
      }
   }
}
