#region File Header
// <copyright company="Bosch Security Systems GmbH">
//
// Authors:
//    Hubert Streidl<Hubert.Streidl@de.bosch.com>
//    Andreas Wetterich
//
// $Revision: 1.6 $
//
// </copyright>
#endregion

using System;
using System.Text;
using System.Diagnostics;
using System.IO;
using System.Runtime.Remoting.Lifetime;


//the base loader namespace
using FSM5000FSILoader;

// API
using FSM5000FSIAPI;

//the api version used in this sample
using FSM5000FSIAPI.Version3;



namespace FSISample3
{

   /// <summary>
   /// The class FSISample3 demonstrates how the loader is used to get a valid library.
   /// We assume the simpliest use case: the panel we want to connect to is
   /// either compatible with the most recent version of the library installed on
   /// this system or there is exactly one version installed witch exactly fits to our
   /// panel network. In both cases we are allowed to use the loader method
   /// GetPreferedLibraryVersion() in order to determine the version to be loaded.   
   /// 
   /// The code which runs FSM5000FSI is nearly identical with FSISample1 but we moved
   /// the code to method "Run" in order to seperate it from the code which is on focus in
   /// this sample: the usage of the loader.   
   /// 
   /// Configuration:
   /// Adapt the default config values in Run() according to your own  
   /// network configuration.
   /// (The default values in this sample application correspond to the 
   /// example values in the FPA-5000 Networking Guide document.)
   /// </summary>
   class FSISample3
   {

      /// <summary>
      /// Receive the traces from the FSI and output the traces to console
      /// In real application it is very useful to write the traces to files
      /// for suppport issues
      /// </summary>
      class TraceConsoleWriter : TextWriter
      {
         public override object InitializeLifetimeService()
         {
            ILease lease = (ILease)base.InitializeLifetimeService();
            if (lease.CurrentState == LeaseState.Initial)
            {
               lease.InitialLeaseTime = TimeSpan.Zero;
            }
            return lease;
         }

         public override Encoding Encoding
         {
            get { return Encoding.Default; }
         }

         public override void Write(string message)
         {
            Console.Out.Write(message);
         }

         public override void WriteLine(string message)
         {
            Console.Out.WriteLine(message);
         }
      }

      /// <summary>
      /// The Config Listener receives all information about the configuration 
      /// received from the panel network. The information must be stored in real application
      /// to know the devices from the panel network and the possible commands.
      /// </summary>
      class ConfigListener : MarshalByRefObject, IPanelConfigListener
      {
         public override object InitializeLifetimeService()
         {
            ILease lease = (ILease)base.InitializeLifetimeService();
            if (lease.CurrentState == LeaseState.Initial)
            {
               lease.InitialLeaseTime = TimeSpan.Zero;
            }
            return lease;
         }

         public void SetItemType(ItemType devItemType)
         {
            Console.Out.WriteLine("Item type " + devItemType.FunctionalType + " (" + devItemType.Description + ")");
         }

         public void SetItem(Item devItem)
         {
            Console.Out.WriteLine("Item  (" + devItem.SIID + ", " + devItem.FunctionalType + " (" + devItem.Label + ")");
         }

         public void RemoveItem(SIID address)
         {
            Console.Out.WriteLine("SI " + address + " removed.");
         }
      }

      /// <summary>
      /// The State Listener receive all states for devices send by the panel network.
      /// 
      /// </summary>
      class StateListener : MarshalByRefObject, IStateListener
      {
         public override object InitializeLifetimeService()
         {
            ILease lease = (ILease)base.InitializeLifetimeService();
            if (lease.CurrentState == LeaseState.Initial)
            {
               lease.InitialLeaseTime = TimeSpan.Zero;
            }
            return lease;
         }

         public void SetItemState(ItemState devItemState)
         {
            Console.Out.WriteLine("State " + devItemState.SIID + " = " + devItemState.LogicalState);
         }

         public void SetMPNetTime(DateTime mpNetTime)
         {
            Console.Out.WriteLine("Time is " + mpNetTime + ".");
         }
      }


      static void Main(string[] args)
      {
         // create loader object
         FSM5000FSIBase baseLoader = new FSM5000FSIBase();
            
         try
         {
            // get loader for FSM5000FSI libs implementing API 3
            IFSM5000FSIAPILoader loader = baseLoader.GetAPILoader(@"3");            

            // get the default library version currently installed
            string libVersion = loader.GetPreferedLibraryVersion();

            // load this version (with option to unload it)
            IFSM5000FSIAPI baseCtrl = loader.Load(libVersion, true);

            IFSM5000FSIController controller = baseCtrl as IFSM5000FSIController;

            //use the pointer to run the stuff
            Run(controller);

            //when finished -> unload it
            loader.Unload(baseCtrl);

         }
         catch(SystemException ex)
         {
            Console.Out.WriteLine("Exception: " + ex.Message);
         }         
      }

      static private void Run(IFSM5000FSIController fsi)
      {
         //Create configuration for FSI 
         FSIConfig cfgFSI = new FSIConfig();

         // ! --- start here to replace values with the values from your own configuration ! ---
         cfgFSI.MPNetGroup = 2;
         cfgFSI.MPNetNode = 1;
         cfgFSI.PNA = 33;      
         cfgFSI.LocalIPAddress = new byte[] { 192, 168, 1, 33 };
         cfgFSI.RepositoryLocation = ".\\repository";

         //  AdvancedFSIConfig is necessary for IP-Multicast settings
         AdvancedFSIConfig advancedFSIConfig = new AdvancedFSIConfig();
         advancedFSIConfig.IsIPMulticast = true;
         advancedFSIConfig.MulticastAddress = new byte[] { 239, 192, 0, 1 };
         advancedFSIConfig.PortNumber = 25001;
         advancedFSIConfig.NetstackTraceLevel = SourceLevels.Error;
         cfgFSI.AdvancedFSIConfig = advancedFSIConfig;
         // ! --- finish here to replace values with the values from your own configuration ! ---

         //Register the listeners 
         fsi.AddListener(new ConfigListener());
         fsi.AddListener(new StateListener());

         //Activate tracing
         TraceConsoleWriter outputter = new TraceConsoleWriter();
         fsi.SetTraceLevel(SourceLevels.Warning);  //for support issues please set the level to SourceLevels.Verbose
         fsi.AddTraceTextWriter(outputter);

         //Startup with configuration
         Result result = fsi.Startup(cfgFSI);

         Console.Out.WriteLine("Startup=" + result);

         //user interaction to finish the application
         Console.Out.WriteLine("======================================================================");
         Console.Out.WriteLine("Press e to end the sample application");
         Console.Out.WriteLine("======================================================================");

         ConsoleKeyInfo input;
         do
         {
            input = Console.ReadKey();
         }
         while (input.Key != ConsoleKey.E);


         //shutdown the sample application
         fsi.Shutdown();         
         
      }


   }
}
