#region File Header
// SPDX-FileCopyrightText: 2020 Bosch Sicherheitssysteme GmbH
//
// SPDX-License-Identifier: LicenseRef-AVENAR-1.0

// Authors:
//       Hubert Streidl <Hubert.Streidl@de.bosch.com>
//       Andreas Wetterich
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Runtime.Remoting.Lifetime;

//the base loader namespace
using FSM5000FSILoader;

// API
using FSM5000FSIAPI;

// the API version which is desired for this sample program
using FSM5000FSIAPI.Version3;          // API v3
// using FSM5000FSIAPI.Version2;       // API v2



namespace FSISample4
{
   /// <summary>
   /// Techniques to be demonstrated in this sample:
   ///
   /// Sample FSISample4 is another loader example with a more complex use case that can be described as follows:
   ///
   /// -	The client knows that it has to connect to a panel network that is compatible to a perticular
   ///    FSM5000FSI Library version, e.g FSI v2.0.15.
   ///    The FSI library version 2.0.x corresponds with API version 3 while the FSI library version 1.x corresponds
   ///    with API version 2.
   /// -  For backward compatibility the FSI Library version 2.0.x DLL-Loader can load API version 2 DLL's as well.
   ///    This can also be tested with this sample program.
   /// -  This sample program is not designed to dynamically select a FSI Library version. The decision is taken at
   ///    compile time as the following is hard coded:
   ///    * namespace FSM5000FSIAPI.Version3 or FSM5000FSIAPI.Version2
   ///    * RequiredAPIID
   ///    * FSI Library ID, e.g. "2.0.15.1"
   ///    Default implementation for this sample is for loading API v3, this can of course be changed for own test
   ///    scenarios.
   ///    For a productive client the implementation needs to be extended in a way that the decision about the FSI
   ///    Library version and its related API version to be loaded can be taken at runtime.
   ///
   /// -	The client can print useful error or warning messages for one of the following scenarios:
   ///    *	The given library is present but does not support the required API
   ///    *	The given library is not present (must be installed)
   /// 	*  The API supported by the selected FSM5000FSI is newer than the API supported by the client. A warning
   /// 	shows, that an update of the SDK is available (request to update the client).
   ///
   /// Except for the more complex loading scenario that shows some additional features of the loader interface the
   /// functionality is nearly identical to sample 3.
   ///
   /// Pre-conditions to build and run the sample:
   ///
   /// - Copy the content of the SDKs \Bin directory into the .\sdk directory to run the sample.
   /// - If necessary remove all versions of FSM5000FSI from the directory except the one that fits to the version
   ///   of your test panel.
   /// - Adopt the references to FSM5000FSILoader and FSM5000FSIAPI in the project file (both files located in
   ///   \sdk after copying)
   /// - Change the hardcoded settings to match your test configuration.
   /// - If you want to have more detailed trace output you can change the line
   ///   fsi.SetTraceLevel(SourceLevels.Warning);
   ///   into
   ///   fsi.SetTraceLevel(SourceLevels.Verbose);
   /// </summary>

   public class Connection
   {
      private string RequiredAPIID;

      private FSM5000FSIBase baseLoader;

      private IFSM5000FSIAPILoader apiLoader;

      private IFSM5000FSIController controller;

      string lastErrorMessage;

      string lastWarningMessage;

      private bool bIsError;
      public bool IsNewErrorMessage
      {
         get { return bIsError;}
      }

      private bool bIsWarning;
      public bool IsNewWarningMessage
      {
         get { return bIsWarning;}
      }

      public string LastWarning
      {
         get
         {
            string str = @"";
            if (IsNewWarningMessage)
            {
               str = lastWarningMessage;
               bIsWarning = false;
            }
            return str;
         }

         internal set
         {
            bIsWarning = true;
            lastWarningMessage = value;
         }
      }

      public string LastError
      {
         get
         {
            string str = @"";
            if (IsNewErrorMessage)
            {
               str = lastErrorMessage;
               bIsError = false;
            }
            return str;
         }

         internal set
         {
            bIsError = true;
            lastErrorMessage = value;
         }
      }


      public IFSM5000FSIController Controller
      {
         get{ return controller; }
      }

      public Connection()
      {
         this.bIsError     = false;
         this.bIsWarning   = false;

         // set this value according to the desired API version which shall be loaded by this sample program
         RequiredAPIID = @"3";            // API v3
         // RequiredAPIID = @"2";         // API v2

         baseLoader = new FSM5000FSIBase();
      }

      public bool Connect(string libraryVersion)
      {
         bool bConnected = false;
         bool bLibFound          =  false;
         bool bLibImplementsAPI  =  false;
         bool bSDKUptoDate       =  false;
         UInt16 clientAPI   =  0;
         UInt16 serverAPI   =  0;

         //disconnect if still connected
         Disconnect();

         foreach (string apiID in baseLoader.APIVersions)
         {
            IFSM5000FSIAPILoader loader = baseLoader.GetAPILoader(apiID);

            IEnumerable<string> libVersions = loader.GetLibraryVersions();
            foreach(string libID in libVersions)
            {
               if(libID == libraryVersion)
               {
                  bLibFound = true;
                  string strOrdinal = loader.GetAPIPropertyValue(@"OrdinalNumber");
                  serverAPI = Convert.ToUInt16(strOrdinal);

                  if (apiID == RequiredAPIID)
                  {
                     IFSM5000FSIAPI baseCtrl = loader.Load(libraryVersion, true);

                     if(baseCtrl is IFSM5000FSIController)
                     {
                        bLibImplementsAPI = true;
                        controller = baseCtrl as IFSM5000FSIController;
                     }
                  }
               }
            }

            if (apiID == RequiredAPIID)
            {
               string strOrdinal = loader.GetAPIPropertyValue(@"OrdinalNumber");
               clientAPI = Convert.ToUInt16(strOrdinal);
            }
         }

         if(clientAPI < serverAPI)
         {
            bSDKUptoDate = false;
         }
         else
         {
            bSDKUptoDate = true;
         }

         //Create message if required
         if(false == bLibFound)
         {
            LastError = @"Library version " + libraryVersion + " not installed!";
         }
         else if(false == bLibImplementsAPI)
         {
            LastError = @"This client does not support library version " + libraryVersion + "!";
         }

         if (false == bSDKUptoDate )
         {
            LastWarning = @"Library version " + libraryVersion + " supports an upgrade of FSM5000FSI!";
         }

         if (controller != null)
         {
            bConnected = true;
         }

         return bConnected;
      }

      public void Disconnect()
      {
         if (null != apiLoader)
         {
            apiLoader.Unload(this.controller);
            controller = null;
            apiLoader = null;
         }
      }
   }

   /// <summary>
   /// Receive the traces from the FSI and output the traces to console
   /// In real application it is very useful to write the traces to files
   /// for suppport issues
   /// </summary>
   class TraceConsoleWriter : TextWriter
   {
      public override object InitializeLifetimeService()
      {
         ILease lease = (ILease)base.InitializeLifetimeService();
         if (lease.CurrentState == LeaseState.Initial)
         {
            lease.InitialLeaseTime = TimeSpan.Zero;
         }
         return lease;
      }

      public override Encoding Encoding
      {
         get { return Encoding.Default; }
      }

      public override void Write(string message)
      {
         Console.Out.Write(message);
      }

      public override void WriteLine(string message)
      {
         Console.Out.WriteLine(message);
      }
   }

   /// <summary>
   /// The Config Listener receives all information about the configuration
   /// received from the panel network. The information must be stored in real application
   /// to know the devices from the panel network and the possible commands.
   /// </summary>
   class ConfigListener : MarshalByRefObject, IPanelConfigListener
   {
      public override object InitializeLifetimeService()
      {
         ILease lease = (ILease)base.InitializeLifetimeService();
         if (lease.CurrentState == LeaseState.Initial)
         {
            lease.InitialLeaseTime = TimeSpan.Zero;
         }
         return lease;
      }

      public void SetItemType(ItemType devItemType)
      {
         Console.Out.WriteLine("Item type " + devItemType.FunctionalType + " (" + devItemType.Description + ")");
      }

      public void SetItem(Item devItem)
      {
         Console.Out.WriteLine("Item  (" + devItem.SIID + ", " + devItem.FunctionalType + " (" + devItem.Label + ")");
      }

      public void RemoveItem(SIID address)
      {
         Console.Out.WriteLine("SI " + address + " removed.");
      }
   }

   /// <summary>
   /// The State Listener receive all states for devices send by the panel network.
   ///
   /// </summary>
   class StateListener : MarshalByRefObject, IStateListener
   {
      public override object InitializeLifetimeService()
      {
         ILease lease = (ILease)base.InitializeLifetimeService();
         if (lease.CurrentState == LeaseState.Initial)
         {
            lease.InitialLeaseTime = TimeSpan.Zero;
         }
         return lease;
      }

      public void SetItemState(ItemState devItemState)
      {
         Console.Out.WriteLine("State " + devItemState.SIID + " = " + devItemState.LogicalState);
      }

      public void SetMPNetTime(DateTime mpNetTime)
      {
         Console.Out.WriteLine("Time is " + mpNetTime + ".");
      }
   }

   class Program
   {
      static void Main(string[] args)
      {
         //create ConnectionLayer
         Connection connection = new Connection();

         // Set here the desired FSI Library Version number to be loaded
         // Note:
         // FSI Library version 2.0.x corresponds with API v3, verify if following is coded:
         // - using FSM5000FSIAPI.Version3;
         // - RequiredAPIID = @"3";
         // FSI Library version 1.x corresponds with API v2, verify if following is coded:
         // - using FSM5000FSIAPI.Version2;
         // - RequiredAPIID = @"2";

         bool bConnected = connection.Connect("2.0.15.1");       // API v3 example
         // bool bConnected = connection.Connect("1.4.3.1");     // API v2 example

         if( true == connection.IsNewErrorMessage )
         {
            Console.WriteLine("Error: " + connection.LastError );
         }

         if( true == connection.IsNewWarningMessage )
         {
            Console.WriteLine("Warning: " + connection.LastWarning );
         }

         if( true == bConnected )
         {
            //using the library

            string Version = connection.Controller.GetVersion();
            Console.WriteLine("Version " + Version + " loaded");

            Run(connection.Controller);

            //finished -> disconnect
            connection.Disconnect();
         }
      }

      static private void Run(IFSM5000FSIController fsi)
      {
         //Create configuration for FSI
         FSIConfig cfgFSI = new FSIConfig();

         // ! --- start here to replace values with the values from your own configuration ! ---
         cfgFSI.MPNetGroup = 2;
         cfgFSI.MPNetNode = 1;
         cfgFSI.PNA = 33;
         cfgFSI.LocalIPAddress = new byte[] { 192, 168, 1, 33 };
         cfgFSI.RepositoryLocation = ".\\repository";

         //  AdvancedFSIConfig is necessary for IP-Multicast settings
         AdvancedFSIConfig advancedFSIConfig = new AdvancedFSIConfig();
         advancedFSIConfig.IsIPMulticast = true;
         advancedFSIConfig.MulticastAddress = new byte[] { 239, 192, 0, 1 };
         advancedFSIConfig.PortNumber = 25001;
         advancedFSIConfig.NetstackTraceLevel = SourceLevels.Error;
         cfgFSI.AdvancedFSIConfig = advancedFSIConfig;
         // ! --- finish here to replace values with the values from your own configuration ! ---

         //Register the listeners
         fsi.AddListener(new ConfigListener());
         fsi.AddListener(new StateListener());

         //Activate tracing
         TraceConsoleWriter outputter = new TraceConsoleWriter();
         fsi.SetTraceLevel(SourceLevels.Warning);  //for support issues please set the level to SourceLevels.Verbose
         fsi.AddTraceTextWriter(outputter);

         //Startup with configuration
         Result result = fsi.Startup(cfgFSI);

         Console.Out.WriteLine("Startup=" + result);

         //user interaction to finish the application
         Console.Out.WriteLine("======================================================================");
         Console.Out.WriteLine("Press e to end the sample application");
         Console.Out.WriteLine("======================================================================");

         ConsoleKeyInfo input;
         do
         {
            input = Console.ReadKey();
         }
         while (input.Key != ConsoleKey.E);


         //shutdown the sample application
         fsi.Shutdown();

      }


   }
}
