#region File Header
// <copyright company="Bosch Security Systems GmbH">
//
// Authors:
//    Hubert Streidl<Hubert.Streidl@de.bosch.com>
//    Andreas Wetterich
//
// $Revision: 1.4 $
//
// </copyright>
#endregion

using System;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Runtime.Remoting.Lifetime;

using FSM5000FSIAPI;
using FSM5000FSILoader;

//the api version used in this sample
using FSM5000FSIAPI.Version3;

namespace FSISample5
{
   /// <summary>
   /// Receive the traces from the FSI and output the traces to console
   /// In real application it is very useful to write the traces to files
   /// for suppport issues
   /// </summary>
   class TraceTextWriter : TextWriter
   {
      private string FileName;

      private StreamWriter stream;

      public override object InitializeLifetimeService()
      {
         ILease lease = (ILease)base.InitializeLifetimeService();
         if (lease.CurrentState == LeaseState.Initial)
         {
            lease.InitialLeaseTime = TimeSpan.Zero;
         }
         return lease;
      }

      public override Encoding Encoding
      {
         get { return Encoding.Default; }
      }

      public override void Write(string message)
      {
         stream.Write(message);
      }

      public override void WriteLine(string message)
      {
         stream.WriteLine(message);
      }

      public void OpenTraceFile()
      {
         stream = File.CreateText(FileName);
      }

      public void CloseTraceFile()
      {
         stream.Close();
      }

      public override void Flush()
      {
         stream.Flush();
      }

      public TraceTextWriter(string fn)
      {
         this.FileName = fn;
      }
   }


   class Tracer 
   {
      TraceTextWriter trace;

      public TextWriter Writer
      {
         get { return trace; }
      }

      public void StartSession( string name)
      {
         string fn = "Trace " + name + " (Library).txt";
         trace = new TraceTextWriter(fn);
         trace.OpenTraceFile();
      }

      public void StopSession()
      { 
         trace.CloseTraceFile(); 
      }
   }

   /// <summary>
   /// The Config Listener receives all information about the configuration 
   /// received from the panel network. The information must be stored in real application
   /// to know the devices from the panel network and the possible commands.
   /// </summary>
   class ConfigListener : MarshalByRefObject, IPanelConfigListener
   {
      TraceTextWriter trace;

      public override object InitializeLifetimeService()
      {
         ILease lease = (ILease)base.InitializeLifetimeService();
         if (lease.CurrentState == LeaseState.Initial)
         {
            lease.InitialLeaseTime = TimeSpan.Zero;
         }
         return lease;
      }

      public void StartSession( string name)
      {
         string fn = "Trace " + name + " (ConfigListener).txt";
         trace = new TraceTextWriter(fn);
         trace.OpenTraceFile();
      }

      public void StopSession()
      { 
         trace.CloseTraceFile(); 
      }

      public void SetItemType(ItemType devItemType)
      {
         trace.WriteLine("Item type " + devItemType.FunctionalType + " (" + devItemType.Description + ")");
         trace.Flush();
      }

      public void SetItem(Item devItem)
      {
         trace.WriteLine("Item  (" + devItem.SIID + ", " + devItem.FunctionalType + " (" + devItem.Label + ")");
         trace.Flush();
      }

      public void RemoveItem(SIID address)
      {
         trace.WriteLine("SI " + address + " removed.");
         trace.Flush();
      }
   }

   /// <summary>
   /// The State Listener receive all states for devices send by the panel network.
   /// 
   /// </summary>
   class StateListener : MarshalByRefObject, IStateListener
   {
      TraceTextWriter trace;

      public override object InitializeLifetimeService()
      {
         ILease lease = (ILease)base.InitializeLifetimeService();
         if (lease.CurrentState == LeaseState.Initial)
         {
            lease.InitialLeaseTime = TimeSpan.Zero;
         }
         return lease;
      }

      public void StartSession(string name)
      {
         string fn = "Trace " + name + " (StateListener).txt";
         trace = new TraceTextWriter(fn);
         trace.OpenTraceFile();
      }

      public void StopSession()
      {
         trace.CloseTraceFile();
      }

      public void SetItemState(ItemState devItemState)
      {
         trace.WriteLine("State " + devItemState.SIID + " = " + devItemState.LogicalState);
         trace.Flush();
      }

      public void SetMPNetTime(DateTime mpNetTime)
      {
         trace.WriteLine("Time is " + mpNetTime + ".");
         trace.Flush();
      }
   }


   class NetworkConnection 
   {
      IFSM5000FSIController fsi;

      Configurator configurator;

      ConfigListener configListener;

      StateListener stateListener;

      Tracer tracer;

      enum Status { eNotCreated, eUninitialized, eConfigured, eRunning };

      Status st;

      public string StatusDescription
      {
         get
         {
            string output = "Unknown";

            switch (st)
            {
               case Status.eNotCreated:
               { 
                  output = "Not created";
               }
               break;

               case Status.eUninitialized:
                  {
                     output = "Not configurated";
                  }
                  break;

               case Status.eConfigured:
                  {
                     output = "Ready to start";
                  }
                  break;

               case Status.eRunning:
                  {
                     output = "Running";
                  }
                  break;
            }
            return output;
         }
      }

      public NetworkConnection()
      {
         FSM5000FSIBase baseLoader = new FSM5000FSIBase();

         configurator = new Configurator();

         configListener = new ConfigListener();

         stateListener = new StateListener();

         tracer = new Tracer();

         string APIVersion = @"3";

         // get loader for FSM5000FSI libs implementing API 1
         IFSM5000FSIAPILoader loader = baseLoader.GetAPILoader(APIVersion);

         if (null != loader)
         {
            // get the default library version currently installed
            string libVersion = loader.GetPreferedLibraryVersion();

            if (libVersion.Length > 0)
            {
               // load this version (with option to unload it)
               IFSM5000FSIAPI baseCtrl = loader.Load(libVersion, true);

               if (null != baseCtrl)
               {
                  // create loader object
                  fsi = baseCtrl as IFSM5000FSIController;

                  st = Status.eUninitialized;
               }
               else
               {
                  Console.Out.WriteLine("Loading FSI lib failed");
               }
            }
            else
            {
               Console.Out.WriteLine("No libraries found");
            }
         }
         else
         {
            Console.Out.WriteLine("No loader found");
         }
      }


      public void ReadConfig( )
      {
         if (st < Status.eRunning)
         {
            configurator.ReadFromConsole();
            st = Status.eConfigured;
         }
         else
         {
            Console.WriteLine("The connection is up - shutdown required in order to change the configuration");
         }
      }

      public void PrintCurrentConfig( )
      {
         configurator.PrintConfigurationToConsole();
      }

       
      public void Startup( )
      {
         bool bContinue = true;
         if (st < Status.eConfigured)
         {
            Console.Out.WriteLine("No configuration entered yet. Start with default values [y/n]?  ");
            ConsoleKeyInfo info = Console.ReadKey();
            if(info.Key != ConsoleKey.Y)
               bContinue = false;
         }
         else if (st == Status.eRunning)
         {
            Console.Out.WriteLine("This connection is already established.");
            bContinue = false;
         }

         if( bContinue )
         {
            string sessionName = configurator.SessionName;

            configListener.StartSession(sessionName);
            stateListener.StartSession(sessionName);
            tracer.StartSession(sessionName);

            //Register the listeners 
            fsi.AddListener(configListener);
            fsi.AddListener(stateListener);

            //Activate tracing
            fsi.SetTraceLevel(SourceLevels.Verbose);
            fsi.AddTraceTextWriter(tracer.Writer);

            fsi.Startup(this.configurator.Configuration);

            Console.Out.WriteLine("FSI connected!");
            st = Status.eRunning;
         }
         else
         {
            Console.Out.WriteLine("FSI NOT connected!");
         }
      }

      public void Shutdown( )
      {
         if (st == Status.eRunning)
         {
            fsi.Shutdown();

            fsi.RemoveListener(configListener);
            fsi.RemoveListener(stateListener);
            fsi.RemoveTraceTextWriter(tracer.Writer);

            configListener.StopSession();
            stateListener.StopSession();
            tracer.StopSession();

            Console.Out.WriteLine("FSI connection terminated!");
            st = Status.eConfigured;
         }
         else
         {
            Console.Out.WriteLine("FSI is not connected!");
         }
      }
   }
}
